﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.SignalR;
using Microsoft.EntityFrameworkCore;
using VCinemaApi.Models;

namespace VCinemaApi.Hubs
{
    public class VCinemaHub : Hub
    {
        private readonly VCinemaContext _context;

        public VCinemaHub(VCinemaContext context)
        {
            _context = context;
        }

        public override async Task OnConnectedAsync()
        {
            var watcher = new Watcher()
            {
                ConnectionId = Context.ConnectionId,
            };
            await _context.Watchers.AddAsync(watcher);
            await _context.SaveChangesAsync();

            await base.OnConnectedAsync();
        }

        public override async Task OnDisconnectedAsync(Exception exception)
        {
            Watcher watcher = await _context.Watchers.SingleOrDefaultAsync(watcher => watcher.ConnectionId == Context.ConnectionId);
            _context.Watchers.Remove(watcher);
            await _context.SaveChangesAsync();

            await base.OnDisconnectedAsync(exception);
        }

        public async Task GetScreens()
        {
            List<Screen> screens = await _context.Screens.ToListAsync();
            await Clients.Caller.SendAsync("ReceiveScreens", screens);
        }

        public async Task GetScreen(int screenId)
        {
            Screen screen = await _context.Screens.SingleOrDefaultAsync(screen => screen.ScreenId == screenId);
            if (screen == null)
            {
                throw new HubException($"No screen with ID {screenId} found.");
            } 
        }

        public async Task CreateScreen(string name, string source)
        {
            var screen = new Screen()
            {
                Name = name,
                Source = source
            };
            await _context.AddAsync<Screen>(screen);
            await _context.SaveChangesAsync();

            var screens = await _context.Screens.ToListAsync();
            await Clients.All.SendAsync("ReceiveScreens", screens);
        }

        public async Task JoinScreen(int screenId)
        {
            Screen screen = await _context.Screens.SingleOrDefaultAsync(screen => screen.ScreenId == screenId);
            if (screen == null)
            {
                throw new HubException($"No screen with ID {screenId} found.");
            }                

            Watcher watcher = await _context.Watchers.SingleOrDefaultAsync(watcher => watcher.ConnectionId == Context.ConnectionId);
            if (watcher == null)
            {
                throw new HubException($"No watcher with connection ID {Context.ConnectionId} found.");
            }

            watcher.Screen = screen;
            await _context.SaveChangesAsync();

            await Groups.AddToGroupAsync(Context.ConnectionId, screenId.ToString());
            await Clients.Caller.SendAsync("ReceiveScreen", screen);

            /* Send new watcher notification of watchers to screen. */
            await Clients.Group(screenId.ToString()).SendAsync("NewWatcher", watcher);
        }

        public async Task LeaveScreen()
        {
            Watcher watcher = await _context.Watchers.SingleOrDefaultAsync(watcher => watcher.ConnectionId == Context.ConnectionId);
            if (watcher == null)
            {
                throw new HubException($"No watcher with connection ID {Context.ConnectionId} found.");
            }

            int screenId = watcher.Screen.ScreenId;
            if (screenId == 0)
            {
                throw new HubException($"Watcher is not joined to any screens.");
            }

            await Groups.RemoveFromGroupAsync(Context.ConnectionId, screenId.ToString());
        }
    }
}
